# -*- coding: utf-8 -*-
"""Configuration options for Azure discovery migration."""
import ipaddress
import json
import pathlib
import typing as t

import yaml
from pydantic import constr  # Constrained string type.
from pydantic import Field, ValidationError, validator

from dm.core.config import BackupConfig, CIDRList, Tags, _BaseModel, _str

if not hasattr(t, "Literal"):
    from typing_extensions import Literal

    t.Literal = Literal


AzureRegionName = t.Literal[
    "westus",
    "eastus",
    "centralus",
    "westus2",
    "northeurope",
    "westeurope",
    "southeastasia",
    "japaneast",
    "chinaeast2",
    "chinanorth2",
]
CleanupResources = t.Literal[
    "PEERING",
    "VNG_ER",
]


def _default_network() -> CIDRList:
    return [ipaddress.IPv4Network("0.0.0.0/0")]


class TerraformConfig(_BaseModel):
    """Terraform configuration.

    Attributes:
        terraform_output: Absolute path to the TF files created.
        terraform_version: Terraform version in terraform version syntax
        aviatrix_provider: Aviatrix terraform provider version in
            terraform version syntax
        aws_provider: AWS terraform provider version in terraform version syntax
        enable_s3_backend: Generate terraform S3 backend config.
            Default to True.
        module_source:  Override the module source in `vpc.tf`. If this is
            omitted or "", we use the included TF module source.
            Defaults to "".
    """

    DEFAULT_MODULE_SOURCE: t.ClassVar[_str] = "../module_azure_brownfield_spoke_vnet"

    regenerate_common_tf: bool = True
    account_folder: t.Literal["name", "id"]
    terraform_output: _str
    terraform_version: _str  # = ">= 0.14"
    aviatrix_provider: _str  # "= 2.19.3"
    aws_provider: _str  # "~> 3.43.0"
    arm_provider: _str  # "~> 2.89.0"
    enable_s3_backend: bool = False
    module_source: t.Optional[_str] = DEFAULT_MODULE_SOURCE


class AviatrixConfig(_BaseModel):
    """Aviatrix config for onboarding.

    Attributes:
        controller_ip: Aviatrix controller IP address.
    """

    controller_ip: ipaddress.IPv4Address


class AlertConfig(_BaseModel):
    """Alert configuration."""

    vnet_name_length: int = 31
    vnet_peering: bool = True
    vcpu_limit: bool = True  # Requires Microsoft.Capacity registration, see README


class ScriptConfig(_BaseModel):
    """Configuration for script features.

    Attributes:
        allow_vnet_cidrs: List of allowed VPC CIDRs. Only the allowed CIDRs will
            be copied to vpc_cidr and passed to the brownfield spoke VPC
            terraform module. Set it to [“0.0.0.0/0”] to allow any CIDR.
        configure_gw_name:
        configure_spoke_advertisement:
        enable_spoke_egress:
        route_table_tags: List of tags to be added to the route table(s). Omit
            this section if no tags required.
            Defaults to [].
        subnet_tags: List of tags to be added to the subnet(s). Omit this
            section if no tags required.
            Defaults to [].
    """

    allow_vnet_cidrs: CIDRList = Field(default_factory=_default_network)
    configure_gw_name: bool = True
    configure_spoke_advertisement: bool = False
    # enable_spoke_egress: bool = False
    route_table_tags: Tags = Field(default_factory=list)
    # subnet_tags: Tags = Field(default_factory=list)


# class TGWConfig(_BaseModel):
#     """Transit gateways.
#
#     Attributes:
#         tgw_account: TGW account number.
#         tgw_role: TGW account access role.
#         tgw_by_region: Dictionary of TGW Regions to TGW ID.
#             e.g.: {"us-east-1": "tgw-020a8339660950770"}
#     """
#
#     tgw_account: _str
#     tgw_role: _str
#     tgw_by_region: t.Dict[AzureRegionName, _str] = Field(default_factory=dict)


GWName = constr(strip_whitespace=True, max_length=29)


class VNetConfig(_BaseModel):
    """Settings for VPC migration.

    Attributes:
    """

    vnet_name: _str
    avtx_cidr: _str
    use_azs: bool = True
    inspection: t.Optional[bool] = None
    spoke_advertisement: t.Optional[t.List[_str]] = None
    spoke_routes: t.Optional[CIDRList] = None
    spoke_gw_name: t.Optional[GWName] = None
    transit_gw_name: t.Optional[GWName] = None
    spoke_gw_tags: Tags = Field(default_factory=list)


class AccountInfo(_BaseModel):
    """Information about spoke VPCs.

    Attributes:
        subscription_id: Azure Subscription ID.
        account_name: Name of the VNet account owner.
        hpe: Enable high performance encryption on spoke gateways.
            Defaults to True.
        filter_cidrs: Filters out any route within specified CIDR when copying
            the route table. No need to add RFC1918 routes in the list; they
            are filtered by default. Set it to empty list [] if no filtering required.
        spoke_gw_size: Spoke gateway instance size.
        add_account:
        onboard_account:
        vnets:
    """

    subscription_id: _str
    account_name: _str
    vnets: t.List[VNetConfig]
    tf_provider_alias: t.Optional[_str] = None
    hpe: bool = True
    filter_cidrs: CIDRList = Field(default_factory=list)
    spoke_gw_size: _str = "Standard_D3_v2"
    onboard_account: bool = False


class AzureCredentials(_BaseModel):
    """Settings used for Azure credentials."""

    arm_directory_id: _str
    arm_application_id: _str
    arm_application_secret_env: _str
    arm_application_secret_data_src: _str
    arm_subscription_id: t.Optional[t.List[_str]]


AzureCredentialConfig = t.Dict[str, AzureCredentials]


class PrestageConfig(_BaseModel):
    """Settings used during prestage."""

    default_route_table: _str


class SwitchTrafficConfig(_BaseModel):
    """Settings used during `switch_traffic.

    Attributes:
        transit_peerings: _str
        default_route_table: _str
        delete_vnet_lock: bool
    """

    transit_peerings: t.Dict[_str, _str] = Field(default_factory=dict)
    delete_vnet_lock: bool = True


class CleanupConfig(_BaseModel):
    """Resources to cleanup.

    Attributes:
        delete_vnet_lock:
        resources: Delete resources like `VGW` or `VIF` in a VPC.
    """

    delete_vnet_lock: bool = True
    resources: t.List[CleanupResources] = Field(default_factory=list)


GW_NAME_KEYS = ["spoke_gw_name", "transit_gw_name"]


class DiscoveryConfiguration(_BaseModel):
    """Discovery Migration Configuration.

    Attributes:
        aviatrix: Generate terraform resource for onboarding an Aviatrix account.
        alert: Alerts configuration.
        config: Script feature configuration.
        tgw: List of TGWs used, assuming all TGWs are defining within one account.
        account_info: Spoke VPC info.
        switch_traffic: Configuration during switch_traffic.
        cleanup: Resources to cleanup.
        aws: Use AWS S3 to backup the generated account folder.
        terraform: Mark the beginning of terraform info.
    """

    label: t.Literal["AZURE", "MGMT_VNET_CIDR"]
    terraform: TerraformConfig
    aviatrix: AviatrixConfig
    azure_cred: AzureCredentialConfig
    account_info: t.List[AccountInfo]
    alert: AlertConfig = Field(default_factory=AlertConfig)
    cleanup: CleanupConfig = Field(default_factory=CleanupConfig)
    config: ScriptConfig = Field(default_factory=ScriptConfig)
    prestage: PrestageConfig = Field(default_factory=PrestageConfig)
    switch_traffic: SwitchTrafficConfig = Field(default_factory=SwitchTrafficConfig)
    aws: t.Optional[BackupConfig] = None
    # tgw: t.Optional[TGWConfig] = None

    @validator("account_info")
    def check_gw_names(cls, account_info, values):
        """Validate gateway names.

        Args:
            val: The account_info dictionary.
            values: All values passed to DiscoveryConfiguration init.

        returns:
            The account_info dictionary.
        """
        errors = []
        if "config" in values:
            configure_gw_names = values["config"].configure_gw_name
            if configure_gw_names:
                for account in account_info:
                    for region in account.regions:
                        for vpc in region.vpcs:
                            if any(getattr(vpc, key) is None for key in GW_NAME_KEYS):
                                errors.append(
                                    (account.account_id, region.region, vpc.vpc_id)
                                )
        if errors:
            error_vpc_str = "\n".join(
                f"account: {account_id}, region: {region}, vpc: {vpc_id}"
                for account_id, region, vpc_id in errors
            )
            raise ValueError(
                "'config.configure_gw_name' is True, both 'spoke_gw_name' and"
                " 'transit_gw_name' must be set in all VPCs."
                "\nList of nonconforming VPCs:\n"
                f"{error_vpc_str}"
            )

        return account_info


def load_from_dict(config_dict: t.Dict) -> DiscoveryConfiguration:
    """Load discovery migration settings from a python dictionary.

    Args:
        config_dict: Python dictionary in which to load configuration
            settings from.

    Returns:
        Parsed discovery migration settings.
    """
    try:
        config = DiscoveryConfiguration(**config_dict)
    except ValidationError as e:
        print(e.json())
        raise SystemExit(1) from e
    return config


def dump_to_dict(config: DiscoveryConfiguration) -> t.Dict:
    """Dump discovery migration settings to a python dictionary.

    Args:
        config: Discovery migration settings.

    Returns:
        Configuration dictionary.
    """
    json_data = config.json()
    data = json.loads(json_data)

    return data


def load_from_yaml(yml_path: pathlib.Path) -> DiscoveryConfiguration:
    """Load discovery migration settings from a yaml.

    Args:
        yml_path: Path to location of discovery migration yaml.

    Returns:
        Parsed discovery migration settings.
    """
    with open(yml_path, "r") as fh:
        data = yaml.load(fh, Loader=yaml.FullLoader)

    return load_from_dict(data)


def dump_to_yaml(config: DiscoveryConfiguration, dest: pathlib.Path) -> pathlib.Path:
    """Dump discovery migration settings to a yaml file.

    Args:
        config: Discovery migration settings.
        dest: Path to destination location of discovery migration yaml.

    Returns:
        Path to destination location of discovery migration yaml.
    """
